/*
 * Copyright (c) 2020 Jetro AS.
 *
 * SPDX-License-Identifier:
 */

#include <zephyr.h>
#include <string.h>
#include <stdio.h>
#include <device.h>
#include <drivers/gps.h>
#include <modem/lte_lc.h>

#include <logging/log.h>
LOG_MODULE_REGISTER(gps);

static const struct device *gps_dev;
static struct gps_config cfg;

#define PVT_MSG_MAX_LEN	256
#define PVT_MSG_FMT	"%d,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d"

static char pvt_msg[PVT_MSG_MAX_LEN];

#define SAT_MSG_MAX_LEN	32
#define SAT_MSG_FMT	"[%d,%d,%d,%d,%d,%d,%d]"
static char sat_msg[SAT_MSG_MAX_LEN * 12];

static void print_pvt(uint32_t fix, struct gps_pvt *pvt)
{
	int len;
	uint32_t sats_in_fix = 0;
	uint32_t sats_in_view = 0;
	uint8_t *sat_msg_ptr = sat_msg;

	/* Count number of satellites used. */
	for (int i=0; i<GPS_PVT_MAX_SV_COUNT; i++) {
		if (pvt->sv[i].in_fix) {
			sats_in_fix++;
		}
		if (pvt->sv[i].sv != 0) {
			sats_in_view++;
		}

		/* Construct SAT message. */
		len = snprintf(sat_msg_ptr, SAT_MSG_MAX_LEN, SAT_MSG_FMT,
			 pvt->sv[i].sv, pvt->sv[i].cn0, pvt->sv[i].elevation,
			 pvt->sv[i].azimuth, pvt->sv[i].signal, pvt->sv[i].in_fix,
			 pvt->sv[i].unhealthy);

		sat_msg_ptr += len;
	}

	/* Construct PVT message. */
	len = snprintf(pvt_msg, PVT_MSG_MAX_LEN, PVT_MSG_FMT,
		       fix, pvt->latitude, pvt->longitude, pvt->altitude, pvt->accuracy,
		       pvt->speed, pvt->heading, pvt->pdop, pvt->hdop, pvt->vdop,
		       pvt->tdop, pvt->gdop, sats_in_fix, sats_in_view);

	LOG_INF("%s", log_strdup(pvt_msg));

	LOG_INF("%s", log_strdup(sat_msg));
}

static void gps_event(const struct device *dev, struct gps_event *evt)
{
	const struct gps_driver_api *gps_api;

	gps_api = (const struct gps_driver_api *)dev->api;

	switch (evt->type) {
		case GPS_EVT_PVT:
			print_pvt(0, &evt->pvt);
			break;
		case GPS_EVT_PVT_FIX:
			print_pvt(1, &evt->pvt);
			break;
		case GPS_EVT_NMEA:
			break;
		case GPS_EVT_NMEA_FIX:
			break;
		case GPS_EVT_AGPS_DATA_NEEDED:
			break;
		default:
			break;
	}
	return;
}

void main(void)
{
	int err;
	bool gps_enabled = false;
	const struct gps_driver_api *gps_api;

	gps_dev = device_get_binding(CONFIG_NRF9160_GPS_DEV_NAME);
	if (!gps_dev) {
		LOG_ERR("Cannot get GPS device");
		return;
	}

	err = lte_lc_system_mode_set(LTE_LC_SYSTEM_MODE_GPS);
	if (err < 0) {
		LOG_ERR("Failed to set system mode: %d", err);
		return;
	}

	err = lte_lc_gps_active();
	if (err < 0) {
		LOG_ERR("Failed to activate GSP: %d", err);
		return;
	}

	gps_api = (const struct gps_driver_api *)gps_dev->api;

	gps_api->init(gps_dev, gps_event);
	cfg.nav_mode = GPS_NAV_MODE_CONTINUOUS;
	cfg.power_mode = GPS_POWER_MODE_DISABLED;

	err = gps_api->start(gps_dev, &cfg);
	if (err == 0) {
		gps_enabled = true;
	}

	while (1) {
		k_sleep(K_SECONDS(60));
	}

	if (gps_enabled) {
		gps_api->stop(gps_dev);
	}

	gps_api->deinit(gps_dev);
}
